/** BEGIN COPYRIGHT BLOCK
 * This Program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 * 
 * This Program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*
 * addindex.c:  Creates one or more indexes for specified attributes
 * 
 * Rob Weltman
 */

#include <stdio.h>
#include <stdlib.h>
#include "libadminutil/admutil.h"
#include "dsalib.h"
#include "init_ds_env.h"
#include <string.h>
#include "nspr.h"

int main(int argc, char *argv[])
{
	int status;
	char *attributes;
	char *attrs;
	char **attrList;
	int nAttrs;
	char *nextAttr = NULL;
	char *backendName;


    fprintf(stdout, "Content-type: text/html\n\n");

	if ( init_ds_env() )
		return 1;

	/*
	 * Get value of the "attributes" variable.
	 */
	attributes = get_cgi_var("attributes", NULL, NULL);
	if ( (NULL == attributes) || (strlen(attributes) < 1) ) {
		rpt_err( DS_UNDEFINED_VARIABLE, "attributes", NULL, NULL );
		return 1;
	}


	backendName = get_cgi_var("backendID", NULL, NULL);
	if ( (NULL == backendName) || (strlen(backendName) < 1) ) {
		rpt_err( DS_UNDEFINED_VARIABLE, "backendName", NULL, NULL );
		return 1;
	}
	

	attrs = strdup( attributes );
	if (attrs == NULL) {
		rpt_err(DS_MEMORY_ERROR, "Out of memory!", NULL, NULL);
		return 1;
	}

	/* Allocate for worst possible case */
	attrList = (char **)malloc(sizeof(*attrList) * (strlen(attrs)+1));
	nAttrs = 0;
	/* strtok() is not MT safe, but it is okay to call here because this is a command line */
	attrList[nAttrs++] = strtok( attrs, " " );
	do {
		nextAttr = strtok( NULL, " " );
		attrList[nAttrs++] = nextAttr;
	} while( nextAttr != NULL );

    ds_send_status((nAttrs > 1) ? "Creating indexes ..." :
				   "Creating index ...");

	status = ds_addindex( attrList, backendName );

	if ( !status ) {
		rpt_success((nAttrs > 1) ? "Success! The indexes have been created." :
					"Success! The index has been created.");
		status = 0;
	} else {
		char msg[BIG_LINE];
		PR_snprintf( msg, sizeof(msg), "[%s] %s", backendName, attributes);
		rpt_err( status, msg, NULL,  NULL );
		status = 1;
	}

	return status;
}
